@@ -1,6 +1,6 @@
 # @file DBI.R
 #
-# Copyright 2018 Observational Health Data Sciences and Informatics
+# Copyright 2019 Observational Health Data Sciences and Informatics
 #
 # This file is part of DatabaseConnector
 # 
@@ -56,15 +56,32 @@ DatabaseConnectorDriver <- function() {
 #' @keywords internal
 #' @export
 #' @import DBI
-#' @import rJava
 setClass("DatabaseConnectorConnection",
          contains = "DBIConnection",
-         slots = list(jConnection = "jobjRef",
-                      identifierQuote = "character",
+         slots = list(identifierQuote = "character",
                       stringQuote = "character",
                       dbms = "character",
                       uuid = "character"))
 
+#' DatabaseConnectorJdbcConnection class.
+#'
+#' @keywords internal
+#' @export
+#' @import rJava
+setClass("DatabaseConnectorJdbcConnection",
+         contains = "DatabaseConnectorConnection",
+         slots = list(jConnection = "jobjRef"))
+
+#' DatabaseConnectorDbiConnection class.
+#'
+#' @keywords internal
+#' @export
+#' @import DBI
+setClass("DatabaseConnectorDbiConnection",
+         contains = "DatabaseConnectorConnection",
+         slots = list(dbiConnection = "DBIConnection",
+                      server = "character"))
+
 #' Create a connection to a DBMS
 #'
 #' @description
@@ -117,10 +134,17 @@ setMethod("show", "DatabaseConnectorConnection", function(object) {
 #' @inherit
 #' DBI::dbIsValid title description params details references return seealso
 #' @export
-setMethod("dbIsValid", "DatabaseConnectorConnection", function(dbObj, ...) {
+setMethod("dbIsValid", "DatabaseConnectorJdbcConnection", function(dbObj, ...) {
   return(!rJava::is.jnull(dbObj@jConnection))
 })
 
+#' @inherit
+#' DBI::dbIsValid title description params details references return seealso
+#' @export
+setMethod("dbIsValid", "DatabaseConnectorDbiConnection", function(dbObj, ...) {
+  return(DBI::dbIsValid(dbObj@dbiConnection))
+})
+
 #' @inherit
 #' DBI::dbQuoteIdentifier title description params details references return seealso
 #' @export
@@ -173,7 +197,7 @@ setClass("DatabaseConnectorResult",
 #' DBI::dbSendQuery title description params details references return seealso
 #' @export
 setMethod("dbSendQuery",
-          signature("DatabaseConnectorConnection", "character"),
+          signature("DatabaseConnectorJdbcConnection", "character"),
           function(conn, statement, ...) {
             if (rJava::is.jnull(conn@jConnection))
               stop("Connection is closed")
@@ -187,6 +211,15 @@ setMethod("dbSendQuery",
             return(result)
           })
 
+#' @inherit
+#' DBI::dbSendQuery title description params details references return seealso
+#' @export
+setMethod("dbSendQuery",
+          signature("DatabaseConnectorDbiConnection", "character"),
+          function(conn, statement, ...) {
+            return(DBI::dbSendQuery(conn@dbiConnection, statement, ...))
+          })
+
 #' @inherit
 #' DBI::dbHasCompleted title description params details references return seealso
 #' @export
@@ -457,10 +490,10 @@ setMethod("dbReadTable",
               name <- paste(database, name, sep = ".")
             }
             sql <- "SELECT * FROM @table;"
-            sql <- SqlRender::renderSql(sql = sql, table = name)$sql
-            sql <- SqlRender::translateSql(sql = sql,
-                                           targetDialect = conn@dbms,
-                                           oracleTempSchema = oracleTempSchema)$sql
+            sql <- SqlRender::render(sql = sql, table = name)
+            sql <- SqlRender::translate(sql = sql,
+                                        targetDialect = conn@dbms,
+                                        oracleTempSchema = oracleTempSchema)
             return(lowLevelQuerySql(conn, sql))
           })
 
@@ -481,10 +514,12 @@ setMethod("dbRemoveTable",
               name <- paste(database, name, sep = ".")
             }
             sql <- "TRUNCATE TABLE @table; DROP TABLE @table;"
-            sql <- SqlRender::renderSql(sql = sql, table = name)$sql
-            sql <- SqlRender::translateSql(sql = sql,
-                                           targetDialect = conn@dbms,
-                                           oracleTempSchema = oracleTempSchema)$sql
-            lowLevelExecuteSql(conn, sql)
+            sql <- SqlRender::render(sql = sql, table = name)
+            sql <- SqlRender::translate(sql = sql,
+                                        targetDialect = conn@dbms,
+                                        oracleTempSchema = oracleTempSchema)
+            for (statement in SqlRender::splitSql(sql)) {
+              lowLevelExecuteSql(conn, statement)
+            }
             return(TRUE)
           })
